from datetime import timedelta

from django.conf import settings
from django.db import transaction
from django.utils import timezone
from greg.api.serializers.role import RoleInvitationSerializer
from greg.models import Identity, Invitation, InvitationLink, Person, Role, Sponsor
from greg.utils import is_identity_duplicate
from gregui.api.serializers.identity import IdentityDuplicateError
from rest_framework import serializers


class InvitationSerializer(serializers.ModelSerializer):
    email = serializers.EmailField(required=True)
    role = RoleInvitationSerializer(required=True)
    sponsor = serializers.CharField(max_length=256)

    def create(self, validated_data):
        email = validated_data.pop("email")
        role_data = validated_data.pop("role")
        sponsor = validated_data.pop("sponsor")

        with transaction.atomic():
            # Find sponsor based on feide_id
            # If not found create sponsor with feide_id instead
            try:
                sponsor = Sponsor.objects.get(feide_id=sponsor)
            except Sponsor.DoesNotExist:
                sponsor = Sponsor.objects.create(feide_id=sponsor)

            person = Person.objects.create(**validated_data)
            Identity.objects.create(
                person=person,
                type=Identity.IdentityType.PRIVATE_EMAIL,
                value=email,
                source=settings.DEFAULT_IDENTITY_SOURCE,
            )
            role_data["person"] = person
            role_data["sponsor"] = sponsor
            role = Role.objects.create(**role_data)
            invitation = Invitation.objects.create(role=role)
            InvitationLink.objects.create(
                invitation=invitation,
                expire=timezone.now() + timedelta(days=30),
            )

        return person

    def validate_email(self, email):
        # The e-mail in the invite is the private e-mail
        if is_identity_duplicate(Identity.IdentityType.PRIVATE_EMAIL, email):
            raise IdentityDuplicateError(
                Identity.IdentityType.PRIVATE_EMAIL, "duplicate_private_email"
            )

        # Return the validated data
        return email

    class Meta:
        model = Person
        fields = (
            "id",
            "first_name",
            "last_name",
            "email",
            "role",
            "sponsor",
        )
